"""Config flow to configure the tinycontrol integration."""

from typing import Any

import voluptuous as vol
from homeassistant.config_entries import ConfigFlow
from homeassistant.const import (
    CONF_HOST,
    CONF_MAC,
    CONF_PASSWORD,
    CONF_MODEL,
    CONF_PORT,
    CONF_USERNAME,
    CONF_SCAN_INTERVAL,
    ATTR_HW_VERSION,
    ATTR_SW_VERSION,
)
from homeassistant.core import callback
from homeassistant.data_entry_flow import FlowResult
from homeassistant.helpers.aiohttp_client import async_get_clientsession
from homeassistant.helpers.device_registry import format_mac

from tinytoolslib.models import async_get_version
from tinytoolslib.exceptions import TinyToolsError, TinyToolsRequestUnauthenticated

from .const import DEFAULT_SCAN_INTERVAL, DOMAIN


class TinycontrolFlowHandler(ConfigFlow, domain=DOMAIN):
    """Handle a tinycontrol device config flow."""

    VERSION = 1

    host: str
    port: int
    username: str
    password: str
    mac: str
    hardware_version: str
    software_version: str
    model: str
    scan_interval: int

    async def async_step_user(
        self,
        user_input: dict[str, Any] | None = None,
    ) -> FlowResult:
        """Handle a flow initiated by the user."""
        if user_input is None:
            return self._async_show_setup_form()

        self.host = user_input[CONF_HOST]
        self.port = user_input[CONF_PORT]
        self.username = user_input[CONF_USERNAME]
        self.password = user_input[CONF_PASSWORD]
        self.scan_interval = user_input[CONF_SCAN_INTERVAL]

        errors = {}
        try:
            await self._get_tinycontrol_mac(raise_on_progress=False)
        except TinyToolsRequestUnauthenticated:
            errors[CONF_USERNAME] = "wrong_credentials"
            errors[CONF_PASSWORD] = "wrong_credentials"
        except TinyToolsError:
            errors["base"] = "cannot_connect"
        if errors:
            return self._async_show_setup_form(errors)

        return self._async_create_entry()

    @callback
    def _async_show_setup_form(
        self, errors: dict[str, str] | None = None
    ) -> FlowResult:
        """Show the setup form to the user."""
        return self.async_show_form(
            step_id="user",
            data_schema=vol.Schema(
                {
                    vol.Required(CONF_HOST): str,
                    vol.Optional(CONF_PORT, default=80): int,
                    vol.Optional(CONF_USERNAME, default=""): str,
                    vol.Optional(CONF_PASSWORD, default=""): str,
                    vol.Optional(
                        CONF_SCAN_INTERVAL,
                        default=int(DEFAULT_SCAN_INTERVAL.total_seconds()),
                    ): vol.All(int, vol.Range(min=1)),
                }
            ),
            errors=errors or {},
        )

    @callback
    def _async_create_entry(self) -> FlowResult:
        return self.async_create_entry(
            title=f"{self.model} ({self.mac}, {self.host}:{self.port})",
            data={
                CONF_MODEL: self.model,
                CONF_HOST: self.host,
                CONF_PORT: self.port,
                CONF_USERNAME: self.username,
                CONF_PASSWORD: self.password,
                CONF_MAC: self.mac,
                ATTR_HW_VERSION: self.hardware_version,
                ATTR_SW_VERSION: self.software_version,
                CONF_SCAN_INTERVAL: self.scan_interval,
            },
        )

    async def _get_tinycontrol_mac(self, raise_on_progress: bool = True) -> None:
        """Get device information from an Tinycontrol device."""
        session = async_get_clientsession(self.hass)
        version_info = await async_get_version(
            self.host,
            self.port,
            self.username,
            self.password,
            with_device=True,
            silent=False,
            session=session,
        )
        tiny_device = version_info["device_model"]
        data = await tiny_device.async_get_all()
        mac = format_mac(data["mac"])
        # Check if already configured
        await self.async_set_unique_id(mac, raise_on_progress=raise_on_progress)
        self._abort_if_unique_id_configured(
            updates={
                CONF_HOST: self.host,
                CONF_PORT: self.port,
                CONF_USERNAME: self.username,
                CONF_PASSWORD: self.password,
                CONF_MAC: mac,
                ATTR_HW_VERSION: version_info["hardware_version"],
                ATTR_SW_VERSION: version_info["software_version"],
                CONF_SCAN_INTERVAL: self.scan_interval,
            }
        )

        self.mac = mac
        self.hardware_version = version_info["hardware_version"]
        self.software_version = version_info["software_version"]
        self.model = tiny_device.info.model
